from spotifyoffline.credentials import getCredentialsFilename, fetchCredentials
from spotifyoffline.spotify import initializeSpotify, getSearchableTracks
from spotifyoffline.youtube import youtubeURLFromTrackName, downloadYoutubeVideoFromURL
from spotifyoffline.conversion import convertMP3ToMP4

from os import getcwd
from os.path import join
from shutil import rmtree

def main() -> None:
    print("[info] clearing \"music/\" directory for new downloads")
    music = join(getcwd(), "music")
    try:
        rmtree(music)
    except FileNotFoundError:
        pass
    try:
        downloaded = 0
        credentialsFilename = getCredentialsFilename()
        credentials = fetchCredentials(credentialsFilename)
        spotify = initializeSpotify(credentials)
        tracks = getSearchableTracks(spotify, credentials["playlist"])
        print("[info] starting download")
        for track in tracks:
            url = youtubeURLFromTrackName(track)
            name = track.replace(" lyrics", "")
            downloadComplete = downloadYoutubeVideoFromURL(name, url)
            filename = name.replace(" ", "-").lower() +".mp4"
            print("[info] converting file type from MP4 to MP3")
            convertMP3ToMP4(filename)
            if downloadComplete:
                downloaded += 1
    except KeyboardInterrupt:
        exit()


from argparse import ArgumentParser
from yaml import safe_load

def getCredentialsFilename() -> str:
    parser = ArgumentParser()
    parser.add_argument("credentials", help = "credentials filename")
    arguments = parser.parse_args()
    return arguments.credentials

def fetchCredentials(credentials: str) -> dict:
    with open(credentials) as credentials:
        credentials = safe_load(credentials)
    return credentials


from moviepy.editor import VideoFileClip

from os import getcwd, remove
from os.path import join

def convertMP3ToMP4(filename: str):
    source = join(getcwd(), "music", filename)
    target = join(getcwd(), "music", filename.replace(".mp4", ".mp3"))
    video = VideoFileClip(source)
    video.audio.write_audiofile(target, logger = None)
    remove(source)


from pytube import YouTube

from requests import get
from re import findall

def youtubeURLFromTrackName(name: str) -> str:
    result = get(f"https://youtube.com/results?search_query={name}")
    result = result.text
    videoIDs = findall(r"watch\?v=(\S{11})", result)
    videoID = videoIDs[0]
    return f"https://youtube.com/watch?v={videoID}"

def downloadYoutubeVideoFromURL(name: str, url: str) -> bool:
    try:
        filename = name.replace(" ", "-").lower() +".mp4"
        YouTube(url).streams.filter(file_extension = "mp4").first().download(filename = filename, output_path = "music")
    except:
        print(f"[error] cannot download video at: \"{url}\"")
        return False
    print(f"[info] downloaded \"{name}\" as \"{filename}\"")
    return True


from spotipy import Spotify
from spotipy.oauth2 import SpotifyClientCredentials

def _performOAuth2(credentials: dict) -> SpotifyClientCredentials:
    apiCredentials = credentials["spotifyAPI"]
    credentialManager = SpotifyClientCredentials(
        apiCredentials["id"],
        apiCredentials["secret"]
    )
    return credentialManager

def initializeSpotify(credentials) -> Spotify:
    spotify = Spotify(client_credentials_manager = _performOAuth2(credentials))
    return spotify

def getSearchableTracks(spotify: Spotify, playlistID: str) -> list:
    searchableTracks = []
    playlist = spotify.playlist(playlistID)
    for track in playlist["tracks"]["items"]:
        track = track["track"]
        name = track["name"]
        artist = track["artists"][0]["name"]
        searchableTracks.append(f"{name} by {artist} lyrics")
    return searchableTracks


from setuptools import setup
from pathlib import Path

code = Path(__file__).parent

readme = (code / "README.md").read_text()
requirements = (code / "requirements.txt")

with open(requirements) as requirements:
    requirements = requirements.readlines()
    requirements = [requirement.replace("\n", "") for requirement in requirements]

setup(
    name = "spotifyoffline",
    version = "v0.0.2",
    description = "listen to your favorite spotify songs, offline",
    long_description = readme,
    long_description_content_type = "text/markdown",
    url = "https://github.com/theaarushgupta/spotifyoffline",
    author = "Aarush Gupta",
    author_email = "aarush@theaarushgupta.com",
    license = "AGPLv3+",
    classifiers = [
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8"
    ],
    packages = ["spotifyoffline"],
    package_data = {},
    install_requires = requirements,
    entry_points = {
        "console_scripts": ["spotifyoffline = spotifyoffline.main:main"]
    }
)